s=io.LoadSurface("../demos/data/sh2","msms")
scene.Add(gfx.Surface("s",s))

# alternative shading mode
scene.SetShadingMode("hf")

# add outlines to surface
scene["s"].Outline(True)
scene["s"].SetOutlineMode(3)

# turn on realtime shadow mapping
scene.SetShadow(True)

scene.SetShadowQuality(3) # default




import math
from ost import geom

filename='fragment.pdb'
if len(sys.argv)>1:
  filename=sys.argv[1]
ent=io.LoadEntity(filename)

edi=ent.EditXCS()
m=geom.Mat4()
m.PasteTranslation(-ent.bounds.min)
edi.ApplyTransform(m)
edi.UpdateICS()
frag=gfx.Entity('frag', ent)
sym_ops=gfx.SymmetryOpList()
for i in range(12):
  m=geom.EulerTransformation(0, i*2.0*math.pi/12.0, 0)
  sym_ops.append(gfx.SymmetryOp(m, geom.Vec3(0, 0, 10)))
  
sym=gfx.SymmetryNode('sym', sym_ops)
scene.Add(sym)
sym.Add(frag)

scene.SetCenter(geom.Vec3())



"""
Superposition of structures made simple.

Authors: Stefan Bienert
"""

import math
import ost.mol.alg
import ost.seq.alg

def ParseAtomNames(atoms):
  """
  Parses different representations of a list of atom names and returns a
  :class:`set`, understandable by :func:`~ost.mol.alg.MatchResidueByNum`. In
  essence, this function translates

  * None to ``None``

  * 'all' to ``None``

  * 'backbone' to ``set(['N', 'CA', 'C', 'O'])``

  * 'aname1, aname2' to ``set(['aname1', 'aname2'])``

  * ``['aname1', 'aname2']``  to ``set(['aname1', 'aname2'])``

  :param atoms: Identifier or list of atoms
  :type atoms: :class:`str`, :class:`list`, :class:`set`
  :returns: A :class:`set` of atoms.
  """
  ## get a set of atoms or None
  if atoms==None:
    return None
  if isinstance(atoms, str):
    if atoms.upper()=='ALL':
      return None
    if atoms.upper()=='BACKBONE':
      return set(['N', 'CA', 'C', 'O'])
    ## if no recognised expression, split at ','
    return set([a.strip() for a in atoms.split(',')])
  return set(atoms)


def _EmptyView(view):
  """
  for internal use, only
  """
  if isinstance(view, ost.mol.EntityHandle):
    return view.CreateEmptyView()
  return view.handle.CreateEmptyView()


def _fetch_atoms(r_a, r_b, result_a, result_b, atmset):
  """
  for internal use, only
  """
  ## compare atoms of residues
  for a_a in r_a.GetAtomList():
    if atmset==None or a_a.name in atmset:
      a_b = r_b.FindAtom(a_a.name)
      if a_b.IsValid():
        result_a.AddAtom(a_a)
        result_b.AddAtom(a_b)
  return result_a, result_b


def _no_of_chains(ent_a, ent_b):
  """
  for internal use, only
  """
  ## get lower no. of chains
  if ent_a.chain_count < ent_b.chain_count:
    return ent_a.chain_count
  return ent_b.chain_count


def MatchResidueByNum(ent_a, ent_b, atoms='all'):
  """
  Returns a tuple of views containing exactly the same number of atoms.
  Residues are matched by residue number. A subset of atoms to be included in
  the views can be specified in the **atoms** argument. Regardless of what the
  list of **atoms** says, only those present in two matched residues will be
  included in the views. Chains are processed in the order they occur in the
  entities. If **ent_a** and **ent_b** contain a different number of chains,
  processing stops with the lower count.

  :param ent_a: The first entity
  :type ent_a: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param ent_b: The second entity
  :type ent_b: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param atoms: The subset of atoms to be included in the two views.
  :type atoms: :class:`str`, :class:`list`, :class:`set`
  :returns: Two :class:`~ost.mol.EntityView` instances with the same amount of
            residues matched by number. Each residue will have the same number
            & type of atoms.
  """
  ## init. final views
  result_a=_EmptyView(ent_a)
  result_b=_EmptyView(ent_b)
  n_chains=_no_of_chains(ent_a, ent_b)
  atmset=ParseAtomNames(atoms)
  ## iterate chains
  for i in range(0, n_chains):
    chain_a=ent_a.chains[i]
    chain_b=ent_b.chains[i]
    residues_a=iter(chain_a.residues)
    ## decide on order of residues
    if chain_a.InSequence() and chain_b.InSequence():
      residues_b=iter(chain_b.residues)
      ## check residues & copy to views
      try:
        while True:
          r_a=next(residues_a)
          r_b=next(residues_b)
          while r_a.number!=r_b.number:
            while r_a.number<r_b.number:
              r_a=next(residues_a)
            while r_b.number<r_a.number:
              r_b=next(residues_b)
          assert r_a.number==r_b.number
          result_a,result_b=_fetch_atoms(r_a, r_b, result_a, result_b, atmset)
      except StopIteration:
        pass
    else:
      ## iterate one list of residues, search in other list
      try:
        while True:
          r_a=next(residues_a)
          r_b=chain_b.FindResidue(r_a.number)
          if r_b.IsValid():
            result_a,result_b=_fetch_atoms(r_a, r_b, result_a, result_b, atmset)
      except StopIteration:
        pass
  result_a.AddAllInclusiveBonds()
  result_b.AddAllInclusiveBonds()
  return result_a, result_b


def MatchResidueByIdx(ent_a, ent_b, atoms='all'):
  """
  Returns a tuple of views containing exactly the same number of atoms.
  Residues are matched by position in the chains of an entity. A subset of
  atoms to be included in the views can be specified in the **atoms** argument.
  Regardless of what the list of **atoms** says, only those present in two
  matched residues will be included in the views. Chains are processed in order
  of appearance. If **ent_a** and **ent_b** contain a different number of
  chains, processing stops with the lower count. The number of residues per
  chain is supposed to be the same.

  :param ent_a: The first entity
  :type ent_a: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param ent_b: The second entity
  :type ent_b: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param atoms: The subset of atoms to be included in the two views.
  :type atoms: :class:`str`, :class:`list`, :class:`set`
  :returns: Two :class:`~ost.mol.EntityView` instances with the same amount of
            residues matched by position. Each residue will have the same number
            & type of atoms.
  """
  not_supported="MatchResidueByIdx has no support for chains of different "\
               +"lengths"
  ## init. final views
  result_a=_EmptyView(ent_a)
  result_b=_EmptyView(ent_b)
  n_chains=_no_of_chains(ent_a, ent_b)
  atmset=ParseAtomNames(atoms)
  ## iterate chains
  for i in range(0, n_chains):
    chain_a=ent_a.chains[i]
    chain_b=ent_b.chains[i]
    ## check equal no. of residues
    if chain_a.residue_count!=chain_b.residue_count:
      raise RuntimeError(not_supported)
    ## iterate residues & copy to views
    for r_a, r_b in zip(chain_a.residues, chain_b.residues):
      result_a,result_b=_fetch_atoms(r_a, r_b, result_a, result_b, atmset)
  result_a.AddAllInclusiveBonds()
  result_b.AddAllInclusiveBonds()
  return result_a, result_b


def _MatchResidueByAln(ent_a, ent_b, atoms, alnmethod):
  """
  For internal use, only
  """
  ## init. final views
  result_a = _EmptyView(ent_a)
  result_b = _EmptyView(ent_b)
  n_chains = _no_of_chains(ent_a, ent_b)
  atmset = ParseAtomNames(atoms)
  ## iterate chains
  for i in range(0, n_chains):
    ## fetch chains (peptide-linking residues only)
    chain_a = ent_a.chains[i]
    chain_b = ent_b.chains[i]
    chain_view_a = chain_a.Select('peptide=true')
    chain_view_b = chain_b.Select('peptide=true')
    if chain_view_a.chain_count == 0 or chain_view_b.chain_count == 0:
      # skip empty chains
      continue
    ## fetch residues
    s_a = ''.join([r.one_letter_code for r in chain_view_a.residues])
    s_b = ''.join([r.one_letter_code for r in chain_view_b.residues])
    ## create sequence from residue lists & alignment
    seq_a = ost.seq.CreateSequence(chain_a.name, s_a)
    seq_b = ost.seq.CreateSequence(chain_b.name, s_b)
    aln_a_b = alnmethod(seq_a, seq_b, ost.seq.alg.BLOSUM62)
    if not aln_a_b:
      # skip failed alignments
      continue
    ## evaluate alignment
    max_aln_res = -1
    for a in range(0, len(aln_a_b)):
      aln = aln_a_b[a]
      aln_res_len = 0
      match_list = list()
      for i in range(0, aln.GetLength()):
        if aln.sequences[0][i]!='-' and aln.sequences[1][i]!='-':
          aln_res_len += 1
          match_list.append(i)
      if aln_res_len > max_aln_res:
        max_aln_res = aln_res_len
        max_aln_idx = a
        max_matches = match_list

    aln = aln_a_b[max_aln_idx]
    ## bind chain to alignment
    aln.AttachView(0, chain_view_a)
    aln.AttachView(1, chain_view_b)
    ## select residues (only replacement edges)
    for i in max_matches:
      r_a = aln.GetResidue(0,i)
      r_b = aln.GetResidue(1,i)
      result_a, result_b = _fetch_atoms(r_a, r_b, result_a, result_b, atmset)
  result_a.AddAllInclusiveBonds()
  result_b.AddAllInclusiveBonds()
  return result_a, result_b

def MatchResidueByLocalAln(ent_a, ent_b, atoms='all'):
  """
  Match residues by local alignment. Takes **ent_a** and **ent_b**, extracts the
  sequences chain-wise and aligns them in Smith/Waterman manner using the
  BLOSUM62 matrix for scoring. Only residues which are marked as :attr:`peptide
  linking <ost.mol.ResidueHandle.peptide_linking>` are considered for alignment.
  The residues of the entities are then matched based on this alignment. Only
  atoms present in both residues are included in the views. Chains are processed
  in order of appearance. If **ent_a** and **ent_b** contain a different number
  of chains, processing stops with the lower count.

  :param ent_a: The first entity
  :type ent_a: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param ent_b: The second entity
  :type ent_b: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param atoms: The subset of atoms to be included in the two views.
  :type atoms: :class:`str`, :class:`list`, :class:`set`
  :returns: Two :class:`~ost.mol.EntityView` instances with the same number of
            residues. Each residue will have the same number & type of atoms.
  """
  return _MatchResidueByAln(ent_a, ent_b, atoms, ost.seq.alg.LocalAlign)

def MatchResidueByGlobalAln(ent_a, ent_b, atoms='all'):
  """
  Match residues by global alignment.
  Same as :func:`MatchResidueByLocalAln` but performs a global Needleman/Wunsch
  alignment of the sequences using the BLOSUM62 matrix for scoring.

  :param ent_a: The first entity
  :type ent_a: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param ent_b: The second entity
  :type ent_b: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param atoms: The subset of atoms to be included in the two views.
  :type atoms: :class:`str`, :class:`list`, :class:`set`
  :returns: Two :class:`~ost.mol.EntityView` instances with the same number of
            residues. Each residue will have the same number & type of atoms.
  """
  return _MatchResidueByAln(ent_a, ent_b, atoms, ost.seq.alg.GlobalAlign)


def Superpose(ent_a, ent_b, match='number', atoms='all', iterative=False,
              max_iterations=5, distance_threshold=3.0):
  """
  Superposes the model entity onto the reference. To do so, two views are
  created, returned with the result. *atoms* describes what goes into these
  views and *match* the selection method. For superposition,
  :func:`SuperposeSVD` or :func:`IterativeSuperposeSVD` are called (depending on
  *iterative*). For matching, the following methods are recognised:

  * ``number`` - select residues by residue number, includes *atoms*, calls
    :func:`MatchResidueByNum`

  * ``index`` - select residues by index in chain, includes *atoms*, calls
    :func:`MatchResidueByIdx`

  * ``local-aln`` - select residues from a Smith/Waterman alignment, includes
    *atoms*, calls :func:`MatchResidueByLocalAln`

  * ``global-aln`` - select residues from a Needleman/Wunsch alignment, includes
    *atoms*, calls :func:`MatchResidueByGlobalAln`

  :param ent_a: The model entity (superposition transform is applied on full
                entity handle here)
  :type ent_a: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`

  :param ent_b: The reference entity
  :type ent_b: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`

  :param match: Method to gather residues/ atoms
  :type match: :class:`str`

  :param atoms: The subset of atoms to be used in the superposition
  :type atoms: :class:`str`, :class:`list`, :class:`set`

  :param iterative: Whether or not to use iterative superpositon.
  :type iterative:  :class:`bool`

  :param max_iterations: Max. number of iterations for
                         :func:`IterativeSuperposeSVD`
                         (only if *iterative* = True)
  :type max_iterations: :class:`int`

  :param distance_threshold: Distance threshold for
                             :func:`IterativeSuperposeSVD`
                             (only if *iterative* = True)
  :type distance_threshold: :class:`float`

  :returns: An instance of :class:`SuperpositionResult`.
  """
  not_supported = "Superpose called with unsupported matching request."
  ## create views to superpose
  if match.upper() == 'NUMBER':
    view_a, view_b = MatchResidueByNum(ent_a, ent_b, atoms)
  elif match.upper() == 'INDEX':
    view_a, view_b = MatchResidueByIdx(ent_a, ent_b, atoms)
  elif match.upper() == 'LOCAL-ALN':
    view_a, view_b = MatchResidueByLocalAln(ent_a, ent_b, atoms)
  elif match.upper() == 'GLOBAL-ALN':
    view_a, view_b = MatchResidueByGlobalAln(ent_a, ent_b, atoms)
  else:
    raise ValueError(not_supported)
  ## action
  if iterative:
    res = ost.mol.alg.IterativeSuperposeSVD(view_a, view_b, max_iterations,
                                            distance_threshold)
  else:
    res = ost.mol.alg.SuperposeSVD(view_a, view_b)
  return res



from ost.gui import trajectory_viewer

"""
MM - Demo

!!!Requires OpenStructure to be compiled with MM-Support!!!

This demo demonstrates how to actually look at the trajectory created
in the gb_example_writing_trajectory.py script.
"""

t = io.LoadCHARMMTraj("gb_example_traj.pdb","gb_example_traj.dcd")
go = gfx.Entity("gb",t.GetEntity())
scene.Add(go)
w = trajectory_viewer.TrajWidget([t],[go])
w.show()



from ost import settings
import subprocess
from xml.dom import minidom
from ost import io, seq
import ost
import os, re, sys

class AlignedPatch:
  """
  An aligned patch, aka. HSP

  .. attribute:: aln

    The local alignment. Sequence offset of both sequences in the alignment are
    set to the starting position in the query and target sequence, respectively.

    :type: :class:`~ost.seq.AlignmentHandle`

  .. attribute:: bit_score

    The bit score of the HSP

  .. attribute:: score

    The score of the HSP

  .. attribute:: evalue

    The E-value of the HSP

  .. attribute:: seqid

    The sequence identity of the HSP
  """
  def __init__(self, aln, bit_score, score, evalue, seqid):
    self.aln=aln
    self.bit_score=bit_score
    self.score=score
    self.evalue=evalue
    self.seqid=seqid

class BlastHit:
  """
  A positive match found by BLAST.
  
  Each blast hit consists of one or more HSPs, encoded by the
  :class:`AlignedPatch` class.
  
  .. attribute:: identifier

    The identifier of the matching sequence

  .. attribute:: aligned_patches

    list of :class:`AlignedPatch` instances holding the actual HSPs.
  """
  def __init__(self, identifier, aligned_patches):
    self.identifier=identifier
    self.aligned_patches=aligned_patches


def ParseBlastOutput(string, seqid_thres=0, evalue_thres=float("infinity")):
  """
  Parses the blast output and returns a list of BlastHits
  setting no seqid_thres or evalue_thres, restores default behaviour without filtering
  """
  def _GetText(node):
    rc=''
    for child in node.childNodes:
        if child.nodeType==child.TEXT_NODE:
            rc+=child.data
    return rc

  def _GetValue(node, tag_name):
    tag=node.getElementsByTagName(tag_name)
    assert len(tag)==1
    return _GetText(tag[0])

  def _GetInt(node, tag_name):
    return int(_GetValue(node, tag_name))

  def _ParseHsp(query_id, hit_id, hsp, tot_query_len, seqid_thres=0, evalue_thres=float("infinity")):
    bit_score=float(_GetValue(hsp, 'Hsp_bit-score'))
    score=float(_GetValue(hsp, 'Hsp_score'))
    evalue=float(_GetValue(hsp, 'Hsp_evalue'))
    try:
      identity=float(_GetValue(hsp, 'Hsp_identity'))
    except AssertionError:
      # The Hsp_identity tag is not a 'must' in the BLAST XML format. It
      # describes the number of matching characters. Hence we assume, if it is
      # missing, there are 0 matches.
      identity=0
    hsp_align_len=float(_GetValue(hsp, 'Hsp_align-len'))
    seqid=identity/hsp_align_len
    query_offset=_GetInt(hsp, 'Hsp_query-from')-1
    hit_offset=_GetInt(hsp, 'Hsp_hit-from')-1
    query_seq=seq.CreateSequence(str(query_id), str(_GetValue(hsp, 'Hsp_qseq')))
    query_seq.offset=query_offset
    hit_seq=seq.CreateSequence(str(hit_id), str(_GetValue(hsp, 'Hsp_hseq')))
    hit_seq.offset=hit_offset
    try:
      if seqid > float(seqid_thres) and evalue < evalue_thres:
        aln=seq.CreateAlignment(query_seq, hit_seq)
        return AlignedPatch(aln, bit_score, score, evalue, seqid)

    except Exception as e:
      print(str(e), query_seq, hit_seq)

  try:
    doc=minidom.parseString(string)
  except Exception as e:
    ost.LogError('Error while parsing BLAST output: %s' % str(e))
    return None
  hits=[]
  query_id=_GetValue(doc, 'BlastOutput_query-def')
  tot_query_len=_GetValue(doc, 'BlastOutput_query-len')
  for hit in doc.getElementsByTagName('Hit'):
    hit_id=_GetValue(hit, 'Hit_def')
    hsps=hit.getElementsByTagName('Hsp')
    aligned_patches=[_ParseHsp(query_id, hit_id, hsp, tot_query_len, seqid_thres, evalue_thres) for hsp in hsps]
    hits.append(BlastHit(hit_id, aligned_patches))
  return hits



class BlastError(RuntimeError):
  def __init__(self, brief, details):
    self.brief=brief
    self.details=details

  def __str__(self):
    if self.details:
      return '%s\n%s' % (self.brief, self.details)
    else:
      return self.brief

def CreateDB(infasta, dbout, mkdb_cmd=None):
  """
  Create a blast DB from a fasta file

  :param infasta: the pdb fasta from which the database will be created
  :type infasta: :class:`string`

  :param dbout: output location for blastDB file
  :type dbout: :class:`string`


  """
  if mkdb_cmd==None:
    try:
      exe=settings.Locate('formatdb')
      args=[exe, '-i', infasta, '-n', dbout]
    except:
      try:
        exe=settings.Locate('makeblastdb')
        args=[exe, '-in', infasta, '-out', dbout, '-dbtype', 'prot']
      except:
        raise RuntimeError('could not find makeblastdb/formatdb executable')
  else:
    if os.path.basename(mkdb_cmd)=='makeblastdb':
      exe=settings.Locate('makeblastdb',explicit_file_name=mkdb_cmd)
      args=[exe, '-in', infasta, '-out', dbout, '-dbtype', 'prot']
    elif os.path.basename(mkdb_cmd)=='formatdb':
        exe=settings.Locate('formatdb',explicit_filename=mkdb_cmd)
        args=[exe, '-i', infasta, '-n', dbout]
    else:
      raise IOError('mkdb command must either be the path to formatdb or makeblastdb!')

  ost.LogInfo('creating blast DB (%s)' % ' '.join(args))
  blast_pipe=subprocess.Popen(args, stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
  blast_pipe.communicate()

def BlastVersion(blast_location=None):
  """
  Returns the version of the BLAST executable, e.g. 2.2.24 as a string
  """

  try:
    blast_exe=settings.Locate('blastall',explicit_file_name=blast_location)
  except:
    try:
      blast_exe=settings.Locate('blastp', explicit_file_name=blast_location)
    except:
      raise RuntimeError('could not find blast executable')

  if os.path.basename(blast_exe)=='blastall':
    args=[blast_exe]
    pattern=re.compile(r'\s*blastall (\d+\.\d+\.\d+)\s+arguments:\s*')

  else:
    args=[blast_exe, '-version']
    pattern=re.compile(r'\s*Package: blast (\d+\.\d+\.\d+),\s+')

  blast_pipe=subprocess.Popen(args, stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
  stdout, _ = blast_pipe.communicate()
  lines=stdout.decode().splitlines()

  for line in lines:
    m=pattern.match(line)
    if m:
      return m.group(1)
  raise IOError("could not determine blast version for '%s'" % blast_exe)



def Blast(query, database, gap_open=11, gap_ext=1, matrix='BLOSUM62',
         blast_location=None, outfmt=0, filter_low_complexity=True):
  """
  Runs a protein vs. protein blast search. The results are returned
  according to the value of the ``outfmt`` parameter.

  :param query: the query sequence
  :type query: :class:`seq.ConstSequenceHandle`

  :param database: The filename of the sequence database. Make sure that
      formatdb has been run on the database and the <database>.pin file exists.
  :param matrix: The substitution matrix to be used. Must be one of 'BLOSUM45',
     'BLOSUM62', 'BLOSUM80', 'PAM30', 'PAM70'.
  :param gap_open: Gap opening penalty. Note that only a subset of gap opening
     penalties is supported for each substitutition matrix. Consult the blast
     docs for more information.
  :param gap_ext: Gap extension penalty. Only a subset of gap extension
     penalties are supported for each of the substitution matrices. Consult the
     blast docs for more information.
  :param outfmt: output format, where '0' corresponds to default output (parsed 
     blast output and 1 to raw string output).
  :param filter_low_complexity: Mask off segments of the query sequence that 
     have low compositional complexity, as determined by the SEG program of 
     Wootton & Federhen (Computers and Chemistry, 1993)
  :rtype: :class:`BlastHit` (with ``outfmt=0``) or :class:`str` 
     (with ``outfmt=1``)
  """
  subst_mats=('BLOSUM45', 'BLOSUM62', 'BLOSUM80', 'PAM30', 'PAM70',)
  if matrix not in subst_mats:
    raise ValueError('matrix must be one of %s' % ', '.join(subst_mats))
  if not os.path.exists('%s.pin' % database) and not os.path.exists('%s.pal' % database):
    raise IOError("Database %s does not exist" % database)
  if blast_location!=None and not os.path.exists(blast_location):
    ost.LogScript('Could not find %s' %blast_location)

  if blast_location==None:
    try:
      blast_exe=settings.Locate('blastall')
    except:
      try:
        blast_exe=settings.Locate('blastp')
      except:
        raise RuntimeError('could not find blast executable')
  else:
    blast_exe=settings.Locate(os.path.basename(blast_location),explicit_file_name=blast_location)

  if os.path.basename(blast_exe)=='blastall':
    args=[blast_exe, '-d', database, '-p', 'blastp',
          '-m', '7', '-M', matrix, '-G', str(gap_open), '-E', str(gap_ext)]
    if filter_low_complexity==False:
      args.append('-F')
      args.append('F')
          
  else:
    complexity_opt='-seg'
    if filter_low_complexity==True:
      complexity_arg='yes'
    else:
      complexity_arg='no'
    args=[blast_exe, '-db', database, '-matrix', matrix,
          '-gapopen', str(gap_open), '-gapextend', str(gap_ext), '-outfmt', '5', complexity_opt, complexity_arg ]

  ost.LogInfo('running BLAST (%s)' % ' '.join(args))
  blast_pipe=subprocess.Popen(args, stderr=subprocess.PIPE,
                              stdout=subprocess.PIPE, stdin=subprocess.PIPE)
  if isinstance(query, str):
    stdout, stderr=blast_pipe.communicate(query.encode())
  else:
    stdout, stderr=blast_pipe.communicate(io.SequenceToString(query, 'fasta').encode())

  if len(stderr)>0:
     pattern=re.compile(r'^\[.*\]\s+ERROR:\s+(.*)')
     lines=stderr.decode().split('\n')
     error_message=pattern.match(lines[0])
     if error_message:
       raise BlastError(error_message.group(1), '\n'.join(lines[1:]))
  if outfmt==0:
    return ParseBlastOutput(stdout.decode())
  else:
    return stdout.decode()



eh=io.LoadEntity("fragment.pdb")
go=gfx.Entity("test",eh.CreateFullView())
ev = eh.Select(mol.Query("rnum=2,3,8,13"))
go.SetSelection(ev)
go.SetRenderMode(gfx.RenderMode.CUSTOM)
go.SetColor(gfx.YELLOW,"(ele=C and rnum=1:4)")
scene.Add(go)
scene.SetCenter(go.GetCenter())

scene.AutoslabMax()



from PyQt5 import QtCore, QtGui, QtWidgets

import os
import ost
from ost import gui

LOGO_PATH = os.path.join(ost.GetSharedDataPath(), "gui", "images", "logo-small.png")


SPLASH_TEXT=""""Welcome to <b>Openstructure</b>!<br/><br/>
You are running version %s<br /><br />If you are new to OpenStructure, we 
invite you to run the demos from the examples directory. Scripts can be 
displayed by right clicking on the file and selecting 'Show source'.<br/><br/>
Feel free visit our website at:<br /> 
<a href='https://www.openstructure.org'>https://www.openstructure.org</a>
""" % ost.VERSION

class SplashDialog(QtWidgets.QDialog):
  def __init__(self, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    layout = QtWidgets.QHBoxLayout(self)
    self.setLayout(layout)
    imageLabel = QtWidgets.QLabel();
    self.pix_map = QtGui.QPixmap(LOGO_PATH);
    imageLabel.setPixmap(self.pix_map);
    layout.addWidget(imageLabel)
    self.label = QtWidgets.QTextBrowser()
    self.label.setReadOnly(True)
    self.label.setOpenExternalLinks(True)
    self.label.setHtml(SPLASH_TEXT)
    layout.addWidget(self.label)
    
def _InitSplash():
  splash = SplashDialog(gui.GostyApp.Instance().perspective.main_area.qobject)
  splash.exec_()

  #QtCore.QTimer.singleShot(30000, splash.close);


__all__=('SplashDialog',)



import unittest,os
from ost.db import LinearIndexer
from ost.db import LinearCharacterContainer
from ost.db import LinearPositionContainer
from ost import geom


def ComparePosList(l1, l2):
  if len(l1)!= len(l2):
    return False
  eps = 0.0001
  for i in range(len(l1)):
    if geom.Distance(l1[i],l2[i]) > eps:
      return False
  return True


class TestLinearDB(unittest.TestCase):

  def testLinearIndexer(self):

    indexer = LinearIndexer()
    self.assertEqual(indexer.GetNumResidues(), 0)

    # check whether error is thrown when list parameters do not have
    # same length
    self.assertRaises(Exception, indexer.AddAssembly, "HELLOU",
                      ["AA", "B"], [20, 33, 1])

    # correct it and add first assembly
    indexer.AddAssembly("HELLOU", ["AA", "B", "CCC"], [20, 33, 1])

    # check whether according entry has been added
    self.assertEqual(indexer.GetAssemblies(), ["HELLOU"])
    self.assertEqual(indexer.GetNumResidues(), 54)

    # check whether error get thrown when the same assembly is added again
    # and directly add another assembly
    self.assertRaises(Exception, indexer.AddAssembly, "HELLOU",  
                      ["AA", "B", "CCC"], [20, 33, 1])

    indexer.AddAssembly("HELLOUYOU", ["X", "Y", "ZZZ"], [10, 26, 20])

    # check whether according entry has been added. we need to sort the
    # output since the GetAssemblies function gives no guarantee about ordering
    entries = indexer.GetAssemblies()
    entries.sort()
    self.assertEqual(entries, ["HELLOU", "HELLOUYOU"])
    self.assertEqual(indexer.GetNumResidues(), 110)

    # check the GetDataRange functions and all their errors when 
    # inexistent data is accessed
    self.assertRaises(Exception, indexer.GetDataRange, "HELL", "CCC")
    self.assertRaises(Exception, indexer.GetDataRange, "HELLOU", "CC")
    self.assertEqual(indexer.GetDataRange("HELLOU", "CCC"), (53,54))

    self.assertRaises(Exception, indexer.GetDataRange, "HELL")
    self.assertEqual(indexer.GetDataRange("HELLOU"), (0,54))

    # add some more assemblies to test proper removal
    indexer.AddAssembly("THIRDENTRY", ["X"], [5])
    indexer.AddAssembly("FOURTHENTRY", ["R","asdf"],[22, 10])

    # check the errors to be thrown when the RemoveAssembly function is called
    self.assertRaises(Exception, indexer.RemoveAssembly, "HELLOUYO")

    # before removing anything we check tons of ranges, we expect
    # them to be ordered as they have been added to the indexer
    self.assertEqual(indexer.GetDataRange("HELLOU"), (0, 54))
    self.assertEqual(indexer.GetDataRange("HELLOU", "B"), (20, 53))    
    self.assertEqual(indexer.GetDataRange("HELLOUYOU"), (54, 110))
    self.assertEqual(indexer.GetDataRange("HELLOUYOU", "Y"), (64, 90))    
    self.assertEqual(indexer.GetDataRange("THIRDENTRY"), (110, 115))
    self.assertEqual(indexer.GetDataRange("THIRDENTRY", "X"), (110, 115))  
    self.assertEqual(indexer.GetDataRange("FOURTHENTRY"), (115, 147))
    self.assertEqual(indexer.GetDataRange("FOURTHENTRY", "asdf"), (137, 147))  

    self.assertEqual(indexer.GetNumResidues(), 147)    

    # remove an assembly
    indexer.RemoveAssembly("HELLOUYOU")

    # we expect, that the data ranges for assembly HELLOU stays the same,
    # but for all others there should be a decrease of 10+26+20=56
    self.assertEqual(indexer.GetDataRange("HELLOU"), (0, 54))
    self.assertEqual(indexer.GetDataRange("HELLOU", "B"), (20, 53))      
    self.assertEqual(indexer.GetDataRange("THIRDENTRY"), (110 - 56, 115 - 56))
    self.assertEqual(indexer.GetDataRange("THIRDENTRY", "X"), (110 - 56, 115 - 56))  
    self.assertEqual(indexer.GetDataRange("FOURTHENTRY"), (115 - 56, 147 - 56))
    self.assertEqual(indexer.GetDataRange("FOURTHENTRY", "asdf"), (137 - 56, 147 - 56))  
    self.assertEqual(indexer.GetNumResidues(), 147 - 56)

    # We additionally check, whether the thing is really gone
    current_entries = indexer.GetAssemblies()
    current_entries.sort()
    self.assertEqual(current_entries, ["FOURTHENTRY", "HELLOU", "THIRDENTRY"])

    # Let's finally test saving and loading
    indexer.Save("test_indexer.dat")
    loaded_indexer = LinearIndexer.Load("test_indexer.dat")
    self.assertEqual(indexer, loaded_indexer)

    if os.path.exists("test_indexer.dat"):
        os.remove("test_indexer.dat")


  def testCharacterContainer(self):

    container = LinearCharacterContainer()

    self.assertEqual(container.GetNumElements(), 0)

    # define a few sequences to play with
    s1 = "AAAAAAA"
    s2 = "BBB"
    s3 = "CCCCCCCCCC"

    # add some stuff
    container.AddCharacters(s1)
    container.AddCharacters(s2)
    container.AddCharacters(s3)

    # check whether stuff has properly been added
    self.assertEqual(container.GetNumElements(), len(s1) + len(s2) + len(s3))
    self.assertEqual(container.GetCharacters((0, container.GetNumElements())), 
                     s1 + s2 + s3)
    self.assertEqual(container.GetCharacters((0, len(s1))), "AAAAAAA")
    self.assertEqual(container.GetCharacters((len(s1), len(s1) + len(s2))), "BBB")
    self.assertEqual(container.GetCharacters((len(s1) + len(s2), 
                                              len(s1) + len(s2) + len(s3))), 
                                              "CCCCCCCCCC")


    # check whether errors get thrown for invalid ranges
    self.assertRaises(Exception, container.GetCharacters, (5,5))
    self.assertRaises(Exception, container.GetCharacters, (5,4))
    self.assertRaises(Exception, container.GetCharacters, (5,100))
    self.assertRaises(Exception, container.GetCharacters, (100,120))
    self.assertRaises(Exception, container.ClearRange, (5,5))
    self.assertRaises(Exception, container.ClearRange, (5,4))
    self.assertRaises(Exception, container.ClearRange, (5,100))
    self.assertRaises(Exception, container.ClearRange, (100,120))

    # let's remove s2 and then check whether the content is as expected
    container.ClearRange((len(s1),len(s1) + len(s2)))
    self.assertEqual(container.GetNumElements(), len(s1) + len(s3))
    self.assertEqual(container.GetCharacters((0, len(s1))), "AAAAAAA")
    self.assertEqual(container.GetCharacters((len(s1), len(s1) + len(s3))), "CCCCCCCCCC")

    # let's check the single character extraction function
    full_expected_string = s1 + s3
    for i in range(container.GetNumElements()):
      self.assertEqual(full_expected_string[i], container.GetCharacter(i))

    # save / load and check for equality
    container.Save("test_character_container.dat")
    loaded_container = LinearCharacterContainer.Load("test_character_container.dat")
    self.assertEqual(container, loaded_container)

    if os.path.exists("test_character_container.dat"):
        os.remove("test_character_container.dat")


  def testPositionContainer(self):

    container = LinearPositionContainer()

    self.assertEqual(container.GetNumElements(), 0)

    # define a few position lists to play with
    pos_1 = geom.Vec3List()
    pos_2 = geom.Vec3List()
    pos_3 = geom.Vec3List()

    pos_1.append(geom.Vec3(-10.234, -45.333, -45.338))
    pos_1.append(geom.Vec3(10.234, 45.333, 45.338))
    pos_1.append(geom.Vec3(10.234, 45.333, 45.338))
    pos_1.append(geom.Vec3(10.234, 45.333, 45.338))
    pos_1.append(geom.Vec3(10.234, 45.333, 45.338))
    pos_1.append(geom.Vec3(10.234, 45.333, 45.338))

    pos_2.append(geom.Vec3(50000.34, 32.43, 33.2222))
    pos_2.append(geom.Vec3(-50000.34, -32.43, -33.2222))
    pos_2.append(geom.Vec3(50000.34, 32.43, 33.2222))

    pos_3.append(geom.Vec3(87.909, 65.222, 1000.555))
    pos_3.append(geom.Vec3(87.909, 65.222, 1000.555))

    # get the rounded target values (the accuracy with which they)
    # are stored in the container)
    pos_1_target = geom.Vec3(10.23, 45.33, 45.34)
    pos_2_target = geom.Vec3(50000.3, 32.4, 33.2) # reduced accuracy!
    pos_3_target = geom.Vec3(87.91, 65.22, 1000.56)

    pos_1_target_list = geom.Vec3List()
    pos_2_target_list = geom.Vec3List()
    pos_3_target_list = geom.Vec3List()

    for i in range(6):
      pos_1_target_list.append(pos_1_target)
    pos_1_target_list[0] = -pos_1_target_list[0]

    for i in range(3):
      pos_2_target_list.append(pos_2_target)
    pos_2_target_list[1] = -pos_2_target_list[1]

    pos_3_target_list.append(pos_3_target)
    pos_3_target_list.append(pos_3_target)

    # add it
    container.AddPositions(pos_1)
    container.AddPositions(pos_2)
    container.AddPositions(pos_3)

    extracted_pos_list = geom.Vec3List()
    extracted_pos = geom.Vec3()

    # check whether stuff has properly been added
    self.assertEqual(container.GetNumElements(), 
                     len(pos_1) + len(pos_2) + len(pos_3))

    container.GetPositions((0,len(pos_1)), extracted_pos_list)

    self.assertTrue(ComparePosList(pos_1_target_list, extracted_pos_list))

    container.GetPositions((len(pos_1),len(pos_1)+len(pos_2)), extracted_pos_list)
    self.assertTrue(ComparePosList(pos_2_target_list, extracted_pos_list))

    container.GetPositions((len(pos_1)+len(pos_2),
                            len(pos_1)+len(pos_2)+len(pos_3)), extracted_pos_list)
    self.assertTrue(ComparePosList(pos_3_target_list, extracted_pos_list))

    # check whether errors get thrown for invalid ranges
    dummy_list = geom.Vec3List()
    dummy_pos = geom.Vec3()
    self.assertRaises(Exception, container.GetPositions, (5,5), dummy_list)
    self.assertRaises(Exception, container.GetPositions, (5,4), dummy_list)
    self.assertRaises(Exception, container.GetPositions, (5,100), dummy_list)
    self.assertRaises(Exception, container.GetPositions, (100,120), dummy_list)
    self.assertRaises(Exception, container.GetPosition, 100, dummy_pos)
    self.assertRaises(Exception, container.ClearRange, (5,5))
    self.assertRaises(Exception, container.ClearRange, (5,4))
    self.assertRaises(Exception, container.ClearRange, (5,100))
    self.assertRaises(Exception, container.ClearRange, (100,120))

    # lets remove pos_2_list and check, whether the result is as expected
    container.ClearRange((len(pos_1), len(pos_1)+len(pos_2)))
    
    self.assertEqual(container.GetNumElements(), len(pos_1) + len(pos_3))

    container.GetPositions((0,len(pos_1)), extracted_pos_list)
    self.assertTrue(ComparePosList(pos_1_target_list, extracted_pos_list))

    container.GetPositions((len(pos_1), len(pos_1) + len(pos_3)), extracted_pos_list)
    self.assertTrue(ComparePosList(pos_3_target_list, extracted_pos_list))

    # check the single position getter
    pos_13_target_list = geom.Vec3List()
    for item in pos_1_target_list:
      pos_13_target_list.append(item)
    for item in pos_3_target_list:
      pos_13_target_list.append(item)

    eps = 0.00001
    for i in range(len(pos_13_target_list)):
      container.GetPosition(i, extracted_pos)
      self.assertTrue(geom.Distance(extracted_pos, pos_13_target_list[i]) < eps)

    # save / load and check for equality
    container.Save("test_position_container.dat")
    loaded_container = LinearPositionContainer.Load("test_position_container.dat")
    self.assertEqual(container, loaded_container)

    if os.path.exists("test_position_container.dat"):
        os.remove("test_position_container.dat")


if __name__ == "__main__":

  from ost import testutils
  testutils.RunTests()




from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from ost import *
from ost import gui
from ost.gui.init_splash import _InitSplash
from ost.gui.dng import termuse
from ost.gui.dng import superpositiondialog

import sys
class FileMenu(QMenu):
  def __init__(self, parent=None):
    QMenu.__init__(self, parent)
    self.setTitle('File')
    gui.AddMenuAction(self, 'Open', self._Open, shortcut='Ctrl+O')
    gui.AddMenuAction(self, 'Save As...', self._SaveAs, 
                  shortcut='Ctrl+Shift+S', 
                  enabled=gui.OneOf(gfx.Entity))
  def _Open(self):
    filename,_=QFileDialog.getOpenFileName(None, 'Open file','')
    if(QFileInfo(filename).isFile()):
      gui.FileLoader.LoadObject(str(filename))

  def _SaveAs(self):
    sel_node=gui.SceneSelection.Instance().GetActiveNode(0)
    filename=QFileDialog.getSaveFileName(None, 'Save As', 
                                         '%s.pdb' % sel_node.name)
    io.SavePDB(sel_node.view, str(filename))
    
class ClipWidget(QWidget):
  def __init__(self, width=500, height=500):
    QWidget.__init__(self)
    self.setWindowFlags(Qt.Tool)
    l = QGridLayout(self)
    l.addWidget(QLabel("Near"), 0, 0)
    l.addWidget(QLabel("Far"), 1, 0)
    bounds_near = QLineEdit(str(0))
    bounds_near.setValidator(QIntValidator(0, 9999, bounds_near))
    bounds_near.setMaxLength(4)
    bounds_near.setMaximumWidth(50)
    bounds_far = QLineEdit(str(200))
    bounds_far.setValidator(QIntValidator(0, 9999, bounds_far))
    bounds_far.setMaxLength(4)
    bounds_far.setMaximumWidth(50)
    l.addWidget(bounds_near, 0, 1, 2, 1)
    l.addWidget(bounds_far, 0, 3, 2, 1)
    self.near_ = QSlider(Qt.Horizontal)
    self.near_.setMinimum(int(bounds_near.text()))
    self.near_.setMaximum(int(bounds_far.text()))
    self.near_.setValue(int(gfx.Scene().near))
    self.far_ = QSlider(Qt.Horizontal)
    self.far_.setMinimum(int(bounds_near.text()))
    self.far_.setMaximum(int(bounds_far.text()))
    far = int(gfx.Scene().far)
    if far>sys.maxsize:
      far = sys.maxsize
    self.far_.setValue(far)
    self.auto_ = QCheckBox("Continuous Automatic Clipping")
    self.auto_.setChecked(gfx.Scene().GetAutoAutoslab())

    l.addWidget(self.near_, 0, 2)
    l.addWidget(self.far_, 1, 2)
    l.addWidget(self.auto_, 2, 0, 1, 4)
    self.near_.valueChanged.connect(self.SetNear)
    self.far_.valueChanged.connect(self.SetFar)
    self.auto_.stateChanged.connect(self.SetAuto)
    bounds_near.textEdited.connect(self.SetNearBounds)
    bounds_far.textEdited.connect(self.SetFarBounds)

  def SetNear(self, val):
    gfx.Scene().near = val

  def SetFar(self, val):
    gfx.Scene().far = val

  def SetAuto(self, val):
    gfx.Scene().AutoAutoslab(val)
    gfx.Scene().near = int(self.near_.value())
    gfx.Scene().far = int(self.far_.value())

  def SetNearBounds(self, text):
    if text!='':
      self.near_.setMinimum(int(text))
      self.far_.setMinimum(int(text))

  def SetFarBounds(self, text):
    if text!='':
      self.near_.setMaximum(int(text))
      self.far_.setMaximum(int(text))

class ExportSceneDialog(QDialog):
  def __init__(self, width=500, height=500):
    QDialog.__init__(self)
    l=QGridLayout(self)
    l.setColumnMinimumWidth(0, 100)
    l.addWidget(QLabel("Width (px)"), 0, 0)
    l.addWidget(QLabel("Height (px)"), 1, 0)
    self.width_=QLineEdit(str(width))
    self.height_=QLineEdit((str(height)))
    self.width_.setValidator(QIntValidator(50, 3000, self.width_))
    self.height_.setValidator(QIntValidator(50, 3000, self.height_))
    self.opaque_=QCheckBox("Force Opaque Background")
    l.addWidget(self.width_, 0, 1)
    l.addWidget(self.height_, 1, 1)
    l.addWidget(self.opaque_, 2, 1)
    hbox=QHBoxLayout()
    cancel=QPushButton("Cancel")
    cancel.clicked.connect(self.reject)
    hbox.addWidget(cancel)
    export=QPushButton("Export")
    hbox.addWidget(export)
    export.setDefault(True)
    l.addLayout(hbox, 3, 1, 2, 1)
    export.clicked.connect(self.accept)
  @property
  def transparent(self):
    return not self.opaque_.isChecked()

  @property
  def width(self):
    return int(self.width_.text())

  @property
  def height(self):
    return int(self.height_.text())

class SceneMenu(QMenu):
  def __init__(self, parent=None):
    QMenu.__init__(self, parent)
    self.setTitle('Scene')
    self.aboutToShow.connect(self._AboutToShow)
    gui.AddMenuAction(self, 'Background Color', self._SetSceneBackground)
    self.fog_action=gui.AddMenuAction(self, 'Depth Cueing', self._ToggleFog, 
                                  shortcut='Ctrl+Shift+F', checkable=True, 
                                  checked=gfx.Scene().fog)
    gui.AddMenuAction(self, 'Center', self._Center,
                  enabled=gui.ManyOf(gfx.GfxObj))
    gui.AddMenuAction(self, 'Fit To Screen', self._FitToScreen,
                  enabled=gui.OneOf(gfx.Entity))
    gui.AddMenuAction(self, 'Superpose', self._SuperposeDialog,
                      enabled=gui.TwoOf(gfx.Entity))
    gui.AddMenuAction(self, 'Save Snapshot', self._ExportScene)
    gui.AddMenuAction(self, 'Scene Clipping', self._ClipScene, shortcut='Ctrl+Shift+C')
    
  def _ExportScene(self):
    qd=ExportSceneDialog()
    if not qd.exec_():
      return

    options = QFileDialog.Options()
    options |= QFileDialog.DontUseNativeDialog
    filename, _ = QFileDialog.getSaveFileName(self,"Save Snapshot","snapshot.png", "", options=options)

    if filename:
      gfx.Scene().Export(str(filename), qd.width, qd.height, qd.transparent)

  def _ClipScene(self):
    self.cw = ClipWidget()
    self.cw.show()

  def _AboutToShow(self):
    self.fog_action.setChecked(gfx.Scene().fog)
    
  def _Center(self):
    sel=gui.SceneSelection.Instance()
    gfx.Scene().center=sel.GetActiveNode(0).center

  def _SetSceneBackground(self):
    new_color=gui.PickColor(gfx.Scene().bg)
    if new_color:
      gfx.Scene().bg=new_color

  def _ToggleFog(self):
    gfx.Scene().fog=not gfx.Scene().fog
    self.fog_action.setChecked(gfx.Scene().fog)

  def _FitToScreen(self):
    sel=gui.SceneSelection.Instance()
    gfx.FitToScreen(sel.GetActiveNode(0))

  def _SuperposeDialog(self):
    sel=gui.SceneSelection.Instance()
    act_count=sel.GetActiveNodeCount()
    # we now that there have to be 2 gfx.Entities, because of using TwoOf to
    # enable menu entry!
    i = 0;
    gfx_ent_1 = sel.GetActiveNode(i)
    while not isinstance(gfx_ent_1, gfx.Entity):
      i += 1
      gfx_ent_1 = sel.GetActiveNode(i)
    i += 1
    gfx_ent_2 = sel.GetActiveNode(i)
    while not isinstance(gfx_ent_2, gfx.Entity):
      i += 1
      gfx_ent_2 = sel.GetActiveNode(i)
    sd = superpositiondialog.SuperpositionDialog(gfx_ent_1, gfx_ent_2)
    if sd.rmsd_superposed_atoms != None:
      if sd.reference == 0:
        gfx_ent_2.UpdatePositions()
        gfx.Scene().CenterOn(gfx_ent_1)
      else:
        gfx_ent_1.UpdatePositions()
        gfx.Scene().CenterOn(gfx_ent_2)
      LogScript('RMSD: %.3f'%sd.rmsd)  
      LogScript('RMSD superposed atoms: %.3f'%sd.rmsd_superposed_atoms) 
      LogScript('fraction superposed: %.3f'%sd.fraction_superposed)   
    elif sd.rmsd != None:
      if sd.reference == 0:
        gfx_ent_2.UpdatePositions()
        gfx.Scene().CenterOn(gfx_ent_1)
      else:
        gfx_ent_1.UpdatePositions()
        gfx.Scene().CenterOn(gfx_ent_2)
      LogScript('RMSD: %.3f'%sd.rmsd)
    elif sd.superposition_error != None:
      LogScript('Superposition Failed: ' + sd.superposition_error)
    else:
      LogScript('Superposition Failed!')

class WindowMenu(QMenu):
  def __init__(self, parent=None):
    QMenu.__init__(self, parent)
    self.setTitle('Window')
    gosty=gui.GostyApp.Instance()
    self.aboutToShow.connect(self._AboutToShow)
    inspector_visible=gosty.GetWidget('InspectorDialog').isVisible()
    self._gl_win_action=gui.AddMenuAction(self, 'GL Window',  
                                      self._ToggleShowGLWindow, checkable=True,
                                      checked=gosty.gl_win.qobject.isVisible(),
                                      shortcut='Ctrl+G')
    self._inspector_action=gui.AddMenuAction(self, 'Inspector', 
                                         self._ToggleShowInspector,
                                         checkable=True,
                                         checked=inspector_visible,
                                         shortcut='Ctrl+I')
    self.addSeparator()
    self.addMenu(gosty.perspective.panels.menu)
    gui.AddMenuAction(self, 'Reset View', self._ResetView)
  def _AboutToShow(self):
    gosty=gui.GostyApp.Instance()
    self._gl_win_action.setChecked(gosty.gl_win.qobject.isVisible())
    inspector_visible=gosty.GetWidget('InspectorDialog').isVisible()
    self._inspector_action.setChecked(inspector_visible)
    
  def _ToggleShowGLWindow(self):
    gosty=gui.GostyApp.Instance()
    gl_win=gosty.GetGLWin()
    if gl_win and gl_win.qobject.isHidden():
      gl_win.Show()
    else:
      gl_win.Hide()

  def _ToggleShowInspector(self):
    gosty=gui.GostyApp.Instance()
    inspector=gosty.GetWidget('InspectorDialog')
    if inspector and inspector.isHidden():
      inspector.show()
    else:
      inspector.hide()
      
  def _ResetView(self):
    msg_box = QMessageBox()
    msg_box.setWindowTitle("Reset the Panels and Widget");
    msg_box.setIcon(QMessageBox.Question)
    msg_box.setText("Do you really want to reset the Panels and Widgets?");
    msg_box.setStandardButtons(QMessageBox.Yes |
                               QMessageBox.Cancel);
    msg_box.setDefaultButton(QMessageBox.Cancel);
    ret = msg_box.exec_();
    if(ret == QMessageBox.Yes):
      settings = QSettings()
      settings.setValue("restore_settings",QVariant(False))
      info_box = QMessageBox()
      info_box.setStandardButtons(QMessageBox.Ok)
      info_box.setIcon(QMessageBox.Information)
      info_box.setWindowTitle("Restart OpenStructure")
      info_box.setText("You must restart OpenStructure for the changes to take effect!");
      info_box.exec_();

class HelpMenu(QMenu):
  def __init__(self, parent=None):
    QMenu.__init__(self, parent)
    self.setTitle('Help')
    gui.AddMenuAction(self, 'Documentation', self._VisitDocs)
    gui.AddMenuAction(self, 'About', self._ShowAboutDialog)
    if sys.platform=='darwin':
      gui.AddMenuAction(self, 'Install Command Line Tool',
                    termuse.InstallTerminalPrograms)
  def _VisitDocs(self):
    QDesktopServices.openUrl(QUrl("http://www.openstructure.org/docs/"))
  
  def _ShowAboutDialog(self):
    _InitSplash()

def _InitMenu():
  _InitMenu.mbar=gui.GostyApp.Instance().perspective.GetMenuBar()
  file_menu=FileMenu(_InitMenu.mbar)
  scene_menu=SceneMenu(_InitMenu.mbar)
  win_menu=WindowMenu(_InitMenu.mbar)
  help_menu=HelpMenu(_InitMenu.mbar)
  _InitMenu.mbar.addMenu(file_menu)
  _InitMenu.mbar.addMenu(scene_menu)
  _InitMenu.mbar.addMenu(win_menu)
  _InitMenu.mbar.addMenu(help_menu)



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
"""
Utility functions to load secondary structure information from DSSP files
and assign them to entities.

Authors: Pascal Benkert, Marco Biasini
"""

import os
import tempfile,subprocess

from ost import io,mol
from ost import settings


def _SkipHeader(stream):
    line=stream.readline()
    while line:
      if line.strip().find('#')==0:
        return True
      line=stream.readline()
    return False

def _Cleanup(pdb_path, temp_path, entity_saved):
  if entity_saved and os.path.exists(pdb_path):
    os.remove(pdb_path)
  if os.path.exists(temp_path):
    os.remove(temp_path)

def _ExecuteDSSP(path, dssp_bin, temp_dir=None):
  # use of mktemp is a safty problem (use mkstemp and provide file handle to 
  # subsequent process
  temp_dssp_path=tempfile.mktemp(suffix=".out",prefix="dssp", dir=temp_dir)
  dssp_abs_path=settings.Locate(['dsspcmbi','dssp','mkdssp'], env_name='DSSP_EXECUTABLE', 
                                explicit_file_name=dssp_bin)
  if os.path.isdir(dssp_abs_path):
    raise RuntimeError('"%s" is a directory. Specify path to DSSP binary' % dssp_abs_path)
  if not os.access(dssp_abs_path, os.X_OK):
    raise RuntimeError('"%s" is not executable' % dssp_abs_path)

  subprocess.run([dssp_abs_path, path, temp_dssp_path])

  return temp_dssp_path


def _CalcRelativeSA(residue_type, absolute_sa):
  solvent_max_list=[118,317,238,243,183,262,286,154,258,228,
                    243,278,260,271,204,234,206,300,303,216] #TODO: source?
  residue_indices = "ARNDCQEGHILKMFPSTWYV"
  # cysteine bridges are marked with lower-case letters by DSSP. We don't 
  # really care which cysteines are forming covalent bonds, so let's set the 
  # one-letter-code to "C".
  if residue_type.islower():
    residue_type='C'
  if residue_indices.find(residue_type)==-1:
    raise RuntimeError('residue %s is a non-standard residue' %(residue_type))	
  else:
    rel=float(absolute_sa)/(solvent_max_list[residue_indices.find(residue_type)])
  return rel
  

def AssignDSSP(ent, pdb_path="", extract_burial_status=False, tmp_dir=None, 
               dssp_bin=None):
  """
  Assign secondary structure states to peptide residues in the structure. This
  function uses the DSSP command line program.

  If you already have a DSSP output file and would like to assign the secondary 
  structure states to an entity, use :func:`LoadDSSP`.
  
  :param ent: The entity for which the secondary structure should be calculated
  :type ent: :class:`~ost.mol.EntityHandle` or :class:`~ost.mol.EntityView`
  :param extract_burial_status: If true, also extract burial status and store
                                as float-property
                                ``relative_solvent_accessibility`` at residue
                                level
  :param tmp_dir: If set, overrides the default tmp directory of the
                  operating system
  :param dssp_bin: The path to the DSSP executable
  :raises: :class:`~ost.settings.FileNotFound` if the dssp executable is not 
      in the path.
  :raises: :class:`RuntimeError` when dssp is executed with errors
  """
  entity_saved = False
  # use of mktemp is a safty problem (use mkstemp and provide file handle to 
  # subsequent process
  pdb_path=tempfile.mktemp(suffix=".pdb",prefix="temp_entity", 
                           dir=tmp_dir)
  io.SavePDB(ent, pdb_path)
  entity_saved = True

  #TODO: exception handling (currently errors occuring here
  # are handled in the parser LoadDSSP)
  temp_dssp_path=_ExecuteDSSP(pdb_path, dssp_bin)
  if not os.path.exists(temp_dssp_path):
    _Cleanup(pdb_path, temp_dssp_path, entity_saved)
    raise RuntimeError('DSSP output file does not exist.')
  # assign DSSP to entity
  try:
    LoadDSSP(temp_dssp_path, ent, extract_burial_status,
             entity_saved)
  except Exception as e:
    # clean up
    print("Exception in DSSP:", e)
    _Cleanup(pdb_path, temp_dssp_path, entity_saved)
    raise RuntimeError(e)

  # clean up
  #print pdb_path, temp_dssp_path
  _Cleanup(pdb_path, temp_dssp_path, entity_saved)

  return ent



def LoadDSSP(file_name, model, extract_burial_status=False,
             entity_saved=False, calculate_relative_sa=True):
    """
    Loads DSSP output and assigns secondary structure states to the peptidic
    residues.

    If you would like to run dssp *and* assign the secondary structure,
    use :func:`AssignDSSP` instead.

    :param file_name: The filename of the DSSP output file
    :param model: The entity to which the secondary structure states should be
                  assigned
    :param extract_burial_status: If true also calculates burial status of
        residues and assigns it to the burial_status string property.
    :param calculate_relative_sa: If true also relative solvent accessibility and
        and assigns it to the relative_solvent_accessibility float property of
        the residue.
    :param entity_save: Whether the entity was saved.
    """
    if not model.IsValid():
      raise ValueError('model entity is not valid')
    if model.atom_count==0:
      raise ValueError('model entity does not contain any atoms')
    stream=open(file_name)
    if not _SkipHeader(stream):
      stream.close()
      raise RuntimeError('Ill-formatted DSSP file')

    for line in stream:
      num=line[6:10].strip()
      ins_code=line[10].strip()
      chain_name=line[11]
      solvent_accessibility=float(line[34:39].strip())
      #solvent_accessibility=line[34:39].strip()
      amino_acid=line[13]
      #print line


      if isinstance(model,mol.ChainView):
        chain=model
      else:
        chain=model.FindChain(chain_name)
        
      if not chain.IsValid():
        continue
      if num=='':
        continue
      residue=None
      try:
        if ins_code == "":
          residue=chain.FindResidue(mol.ResNum(int(num)))
        else:
          residue=chain.FindResidue(mol.ResNum(int(num),ins_code))

        # set property "burial status:
        if extract_burial_status:
          #set default (dummy) burial status for incomplete residues:
          residue.SetStringProp("burial_status", 'X')

          #handle seleno-methionine appearing as amino acid 'X' in DSSP:
          if residue.name=="MSE" and amino_acid=='X':
            amino_acid='M'

          residue.SetFloatProp("solvent_accessibility",
                                          solvent_accessibility)
          if calculate_relative_sa:
            relative_sa=_CalcRelativeSA(amino_acid,solvent_accessibility)
            residue.SetFloatProp("relative_solvent_accessibility",
                                            relative_sa)
            if relative_sa < 0.25:
              residue.SetStringProp("burial_status", 'b')
            else:
              residue.SetStringProp("burial_status", 'e')
      except Exception as e:
        print("ERROR:",e)
        continue

      rtype=line[16:17]
      rt=mol.SecStructure.COIL
      if rtype=='H':
        rt=mol.SecStructure.ALPHA_HELIX
      elif rtype=='E':
        rt=mol.SecStructure.EXTENDED
      elif rtype=='B':
        rt=mol.SecStructure.BETA_BRIDGE
      elif rtype=='S':
        rt=mol.SecStructure.BEND
      elif rtype=='T':
        rt=mol.SecStructure.TURN
      elif rtype=='I':
        rt=mol.SecStructure.PI_HELIX
      elif rtype=='G':
        rt=mol.SecStructure.THREE_TEN_HELIX
      # for corrupted DSSP files. Catch in calling routine:
      if not residue.IsValid():
        #Todo: if residues with occupancy 0 have been removed before
        #using a selection statement, they are missed here
        #IMPORTANT: asign DSSP before any selections
        stream.close()
        raise RuntimeError('Ill-formatted DSSP file: invalid residue')	
      residue.SetSecStructure(mol.SecStructure(rt))
    stream.close()



from ost import io,mol
import unittest
import os

class TestHBond(unittest.TestCase):
  
  def setUp(self):
    p=io.IOProfile(dialect='CHARMM')
    self.eh1=io.LoadPDB(os.path.join("testfiles","hbond1.pdb"),profile=p)
    self.eh2=io.LoadPDB(os.path.join("testfiles","hbond2.pdb"),profile=p)
    self.hb_da_dict=mol.alg.hbond.BuildCHARMMHBondDonorAcceptorDict()
  
  def testHBondList(self):
    hbl1=mol.alg.hbond.GetHbondListFromView(self.eh1.Select('cname=PROA'),self.hb_da_dict)
    for hb in hbl1:self.assertTrue(hb.IsFormed())
    hbl=mol.alg.hbond.GetHbondListBetweenViews(self.eh1,self.eh2,self.hb_da_dict)
    self.assertEqual(len(hbl),1)
    
  def testHBondScore(self):
    self.assertEqual(mol.alg.hbond.CalculateHBondScore(self.eh1,self.eh2,hbond_donor_acceptor_dict=self.hb_da_dict),0.8)
  

if __name__ == "__main__":
  from ost import testutils
  testutils.RunTests()



from PyQt5 import QtCore
import math
# remove all objects from scene, just in case
scene.RemoveAll()

class Anim(QtCore.QTimer):
    def __init__(self, a, b):
        QtCore.QTimer.__init__(self)
        self.a=a
        self.b=b
        self.angle=0.0
        self.dir=0.01
        self.edi=self.a.view.handle.EditXCS(mol.UNBUFFERED_EDIT)
        self.timeout.connect(self.OnTimer)
        
    def OnTimer(self):
        self.angle+=self.dir
        if self.angle>math.pi/2:
          self.dir=-self.dir
        if self.angle<0:
          self.dir=-self.dir
        rot=geom.AxisRotation(geom.Vec3(0.0, 0.0, -1.0), self.angle)
        self.edi.SetTransform(geom.Mat4(rot))
        self.edi.UpdateICS()
        for a in self.b.view.atoms:
          score=mol.alg.ClashScore(a.handle, self.a.view)
          a.SetFloatProp('clash', score)
        self.a.UpdatePositions()
        self.b.ReapplyColorOps()


def Hammer(off=geom.Vec3()):
  ent=mol.CreateEntity()
  edi=ent.EditXCS(mol.BUFFERED_EDIT)
  chain=edi.InsertChain("A")
  res=edi.AppendResidue(chain, "QUAD")
  a=edi.InsertAtom(res, "A", off+geom.Vec3(0.0, 0.0, 0.0), "H")
  b=edi.InsertAtom(res, "B", off+geom.Vec3(0.0, 4.0, 0.0), "H")
  c=edi.InsertAtom(res, "C", off+geom.Vec3(2.0, 4.0, 0.0), "H")
  d=edi.InsertAtom(res, "D", off+geom.Vec3(2.0, 5.0, 0.0), "H")
  e=edi.InsertAtom(res, "E", off+geom.Vec3(-2.0, 5.0, 0.0), "H")
  f=edi.InsertAtom(res, "F", off+geom.Vec3(-2.0, 4.0, 0.0), "H")
  edi.Connect(a, b)
  edi.Connect(b, c)
  edi.Connect(c, d)
  edi.Connect(d, e)
  edi.Connect(e, f)
  edi.Connect(f, b)    
  return ent

def TheWall():
  ent=mol.CreateEntity()
  edi=ent.EditXCS(mol.BUFFERED_EDIT)
  chain=edi.InsertChain("A")
  res=edi.AppendResidue(chain, "QUAD")
  index=0
  for i in range(-10, 10):
    for j in range(-10, 10):
      index+=1
      atom=edi.InsertAtom(res, "A%d" % index, geom.Vec3(4.0, -2.0, 0.0)+
                          geom.Vec3(i, 0, j)*0.25)
      atom.radius=0.25
                     
  return ent

a=Hammer()
b=TheWall()

a_go=gfx.Entity("a", gfx.CUSTOM, a)
b_go=gfx.Entity("b", gfx.CUSTOM, b)
for a in a.atoms:
  a.SetFloatProp('clash', 0.0)

scene.Add(a_go)
scene.Add(b_go)

anim=Anim(a_go, b_go)
anim.start(20)
grad=gfx.Gradient()
grad.SetColorAt(0.0, gfx.Color(0.0, 1.0, 0.0))
grad.SetColorAt(0.5, gfx.Color(1.0, 0.0, 1.0))
grad.SetColorAt(1.0, gfx.Color(1.0, 0.0, 0.0))
b_go.ColorBy('clash', gfx.Color(0,1,0), gfx.Color(1,0,0), 0.0, 10.0, mol.Prop.Level.ATOM)
scene.CenterOn(b_go)
print('Demo 7: Illustrating a clash score')



# remove all objects from scene, just in case
scene.RemoveAll()
# Load chain A of SDH
ent = io.LoadPDB('data/sdh.pdb', restrict_chains='A')

# create graphics object
go = gfx.Entity('SDH', ent)
# set rendermode of full graphical object to interpolated spline
go.SetRenderMode(gfx.SLINE)
# select sidechains (including bond to C-alpha and N in case of proline)
sidechains = ent.Select('aname!=CA,C,N,O', mol.EXCLUSIVE_BONDS)
# set rendering of sidechains to simple
go.SetRenderMode(gfx.SIMPLE, sidechains)

# add object to scene
scene.Add(go)

# center the scene on the geometric center of the SDH
scene.CenterOn(go)

print('Demo 1: loading and displaying a pdb file')



"""
Some functions for analyzing structures

Author: Niklaus Johner (Niklaus.Johner@unibas.ch)
"""
import os
import ost

def GetFrameFromEntity(eh):
  """
  This function returns a CoordFrame from an EntityHandle
  
  :param eh: 
  :type eh: :class:`~ost.mol.EntityHandle`

  :return: :class:`ost.mol.CoordFrame`
  """
  return ost.mol.CreateCoordFrame(eh.GetAtomPosList(ordered_by_index=True))
  
def GetDistanceBetwCenterOfMass(sele1,sele2):
  """
  This function calculates the distance between the centers of mass
  of **sele1** and **sele2**, two selections from the same Entity.

  :param sele1:
  :param sele2:
  :type sele1: :class:`~ost.mol.EntityView`
  :type sele2: :class:`~ost.mol.EntityView`

  :return: :class:`float`
  """
  if not sele1.IsValid() and sele2.IsValid():
    print('invalid view')
    return
  eh=sele1.GetHandle()
  if not eh==sele2.GetHandle():
    print('The two views must be from the same entity')
    return
  f=GetFrameFromEntity(eh)
  return f.GetDistanceBetwCenterOfMass(sele1,sele2)

def GetMinDistanceBetweenViews(sele1,sele2):
  """
  This function calculates the minimal distance between
  **sele1** and **sele2**, two selections from the same Entity.
  
  :param sele1:
  :param sele2:
  :type sele1: :class:`~ost.mol.EntityView`
  :type sele2: :class:`~ost.mol.EntityView`

  :return: :class:`float`
  """
  if not sele1.IsValid() and sele2.IsValid():
    print('invalid view')
    return
  eh=sele1.GetHandle()
  if not eh==sele2.GetHandle():
    print('The two views must be from the same entity')
    return
  f=GetFrameFromEntity(eh)
  return f.GetMinDistance(sele1,sele2)

def GetMinDistBetwCenterOfMassAndView(sele1,sele2):
  """
  This function calculates the minimal distance between **sele2** and
  the center of mass of **sele1**, two selections from the same Entity.

  :param sele1: The selection from which the center of mass is taken
  :param sele2:
  :type sele1: :class:`~ost.mol.EntityView`
  :type sele2: :class:`~ost.mol.EntityView`

  :return: distance (\ :class:`float`\ )
  """
  if not sele1.IsValid() and sele2.IsValid():
    print('invalid view')
    return
  eh=sele1.GetHandle()
  if not eh==sele2.GetHandle():
    print('The two views must be from the same entity')
    return
  f=GetFrameFromEntity(eh)
  return f.GetMinDistBetwCenterOfMassAndView(sele1,sele2)
  

def GetAlphaHelixContent(sele1):
  """
  This function calculates the content of alpha helix in a view.
  All residues in the view have to ordered and adjacent (no gaps allowed)
  
  :param sele1:
  :type sele1: :class:`~ost.mol.EntityView`

  :return: :class:`float`
  """
  if not sele1.IsValid():
    print('invalid view')
    return
  eh=sele1.GetHandle()
  f=GetFrameFromEntity(eh)
  return f.GetAlphaHelixContent(sele1)


def CalculateBestFitLine(sele1):
  """
  This function calculates the best fit line to the atoms in **sele1**.
  
  :param sele1:
  :type sele1: :class:`~ost.mol.EntityView`

  :return: :class:`~ost.geom.Line3`
  """
  if not sele1.IsValid():
    print('invalid view')
    return
  eh=sele1.GetHandle()
  f=GetFrameFromEntity(eh)
  return f.GetODRLine(sele1)

def CalculateBestFitPlane(sele1):
  """
  This function calculates the best fit plane to the atoms in **sele1**.
  
  :param sele1:
  :type sele1: :class:`~ost.mol.EntityView`

  :return: :class:`~ost.geom.Plane`
  """
  if not sele1.IsValid():
    print('invalid view')
    return
  eh=sele1.GetHandle()
  f=GetFrameFromEntity(eh)
  return f.GetODRPlane(sele1)

def CalculateHelixAxis(sele1):
  """
  This function calculates the best fit cylinder to the CA atoms in **sele1**,
  and returns its axis.  Residues should be ordered correctly
  in **sele1**.
  
  :param sele1:
  :type sele1: :class:`~ost.mol.EntityView`

  :return: :class:`~ost.geom.Line3`
  """
  if not sele1.IsValid():
    print('invalid view')
    return
  eh=sele1.GetHandle()
  f=GetFrameFromEntity(eh)
  return f.FitCylinder(sele1)[0]


def CalculateDistanceDifferenceMatrix(sele1,sele2):
  """
  This function calculates the pairwise distance differences between two selections (\ :class:`~ost.mol.EntityView`\ ).
  The two selections should have the same number of atoms
  It returns an NxN DistanceDifferenceMatrix M (where N is the number of atoms in sele1)
  where M[i,j]=||(sele2.atoms[i].pos-sele2.atoms[j].pos)||-||(sele1.atoms[i].pos-sele1.atoms[j].pos)||

  :param sele1:
  :param sele2:
  :type sele1: :class:`~ost.mol.EntityView`
  :type sele2: :class:`~ost.mol.EntityView`

  :return: NxN numpy matrix
  """
  try:import numpy as npy
  except ImportError:
    LogError("Function needs numpy, but I could not import it.")
    raise
  if not sele1.IsValid() and sele2.IsValid():
    print('invalid view')
    return
  if not sele1.GetAtomCount()==sele2.GetAtomCount():
    print('The two views must have the same number of atoms')
    return
  n_atoms=sele1.GetAtomCount()
  M=npy.zeros([n_atoms,n_atoms])
  for i,a1 in enumerate(sele1.atoms):
    for j,a2 in enumerate(sele1.atoms):
      if i>=j:continue
      d1=ost.geom.Distance(a1.pos,a2.pos)
      d2=ost.geom.Distance(sele2.atoms[i].pos,sele2.atoms[j].pos)
      M[i,j]=d2-d1
      M[j,i]=d2-d1
  return M





#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
import traceback
from ._ost_gfx import *

class PyGfxObj(GfxObj):
  def __init__(self,name):
    """
    requires a unique name not yet utilized in the Scene;
    do not place OpenGL calls in the ctor, use InitGL for
    that purpose
    """
    GfxObj.__init__(self,name)
    self._valid_flag=False

  def _InitGL(self):
    try:
      self.InitGL()
      self._valid_flag=True
    except:
      traceback.print_exc()
      
  def InitGL(self):
    """
    called once for each OpenGL context (usually one time),
    allows one-time OpenGL initialization to be implemented,
    such as vbo allocation
    """
    pass


  def _CustomPreRenderGL(self,rebuild):
    if not self._valid_flag:
      return
    try:
      self.CustomPreRenderGL(rebuild)
    except:
      self._valid_flag=False
      traceback.print_exc()

  def CustomPreRenderGL(self,rebuild):
    """
    called just before CustomRenderGL is called; the flag
    indicates that a rebuild is required or was requested
    """
    pass


  def _CustomRenderGL(self,render_pass):
    if not self._valid_flag:
      return
    try:
      self.CustomRenderGL(render_pass)
    except:
      self._valid_flag=False
      traceback.print_exc()

  def CustomRenderGL(self,render_pass):
    """
    called for each scene refresh
    """
    pass




#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
import os, tempfile, ftplib, http.client

from ._ost_io import *
from ost import mol, geom, conop, seq

profiles=None

class IOProfiles:
  def __init__(self):
     self._dict={}

  def __getitem__(self, key):
    return IOProfileRegistry.Instance().Get(key)

  def __setitem__(self, key, value):
    if isinstance(value, str):
      value=self[value].Copy()
    IOProfileRegistry.Instance().Set(key, value)
    self._dict[key]=value

  def __len__(self):
    return len(self._dict)

  def __iter__(self):
    return self._dict.__iter__()

if not profiles:
  profiles=IOProfiles()
  if conop.GetDefaultLib():
    processor = conop.RuleBasedProcessor(conop.GetDefaultLib())
  else:
    processor = conop.HeuristicProcessor()
  profiles['STRICT']=IOProfile(dialect='PDB', fault_tolerant=False,
                               quack_mode=False, processor=processor.Copy())
  profiles['SLOPPY']=IOProfile(dialect='PDB', fault_tolerant=True,
                               quack_mode=True, processor=processor.Copy())
  profiles['CHARMM']=IOProfile(dialect='CHARMM', fault_tolerant=True,
                               quack_mode=False, processor=processor.Copy())
  profiles['DEFAULT']='STRICT'

def _override(val1, val2):
  if val2!=None:
    return val2
  else:
    return val1

def LoadPDB(filename, restrict_chains="", no_hetatms=None,
            fault_tolerant=None, load_multi=False, quack_mode=None,
            join_spread_atom_records=None, calpha_only=None,
            profile='DEFAULT', remote=False, remote_repo='pdb',
            dialect=None, seqres=False, bond_feasibility_check=None):
  """
  Load PDB file from disk and return one or more entities. Several options 
  allow to customize the exact behaviour of the PDB import. For more information 
  on these options, see :doc:`profile`.
  
  Residues are flagged as ligand if they are mentioned in a HET record.

  :param restrict_chains: If not an empty string, only chains listed in the
     string will be imported.

  :param fault_tolerant: Enable/disable fault-tolerant import. If set, overrides 
     the value of :attr:`IOProfile.fault_tolerant`.

  :param no_hetatms: If set to True, HETATM records will be ignored. Overrides 
      the value of :attr:`IOProfile.no_hetatms`

  :param load_multi: If set to True, a list of entities will be returned instead
      of only the first. This is useful when dealing with multi-PDB files.

  :param join_spread_atom_records: If set, overrides the value of 
      :attr:`IOProfile.join_spread_atom_records`.
  
  :param remote: If set to True, the method tries to load the pdb from the 
     remote repository given as *remote_repo*. The filename is then 
     interpreted as the entry id as further specified for the *remote_repo*
     parameter.

  :param remote_repo: Remote repository to fetch structure if *remote* is True. 
                      Must be one in ['pdb', 'smtl', 'pdb_redo']. In case of 
                      'pdb' and 'pdb_redo', the entry must be given as lower 
                      case pdb id, which loads the deposited assymetric unit 
                      (e.g. '1ake'). In case of 'smtl', the entry must also 
                      specify the desired biounit (e.g. '1ake.1').
     
  :rtype: :class:`~ost.mol.EntityHandle` or a list thereof if `load_multi` is 
      True.

  :param dialect: Specifies the particular dialect to use. If set, overrides 
    the value of :attr:`IOProfile.dialect`

  :param seqres: Whether to read SEQRES records. If set to True, the loaded 
    entity and seqres entry will be returned as a tuple.

  :type dialect: :class:`str`
  

  :raises: :exc:`~ost.io.IOException` if the import fails due to an erroneous or 
      inexistent file
  """
  def _override(val1, val2):
    if val2!=None:
      return val2
    else:
      return val1
  if isinstance(profile, str):
    prof=profiles[profile].Copy()
  elif isinstance(profile, IOProfile):
    prof=profile.Copy()
  else:
    raise TypeError('profile must be of type string or IOProfile, '+\
                    'instead of %s'%type(profile))
  if dialect not in (None, 'PDB', 'CHARMM',):
    raise ValueError('dialect must be PDB or CHARMM')
  prof.calpha_only=_override(prof.calpha_only, calpha_only)
  prof.no_hetatms=_override(prof.no_hetatms, no_hetatms)
  prof.dialect=_override(prof.dialect, dialect)
  prof.quack_mode=_override(prof.quack_mode, quack_mode)
  if prof.processor:
    prof.processor.check_bond_feasibility=_override(prof.processor.check_bond_feasibility, 
                                                    bond_feasibility_check)
  prof.fault_tolerant=_override(prof.fault_tolerant, fault_tolerant)
  prof.join_spread_atom_records=_override(prof.join_spread_atom_records,
                                          join_spread_atom_records)

  tmp_file = None # avoid getting out of scope
  if remote:
    if remote_repo not in ['pdb', 'smtl', 'pdb_redo']:
      raise IOError("remote_repo must be in ['pdb', 'smtl', 'pdb_redo']")
    from ost.io.remote import RemoteGet
    tmp_file =RemoteGet(filename, from_repo=remote_repo)
    filename = tmp_file.name
  
  conop_inst=conop.Conopology.Instance()
  if prof.processor:
    if prof.dialect=='PDB':
      prof.processor.dialect=conop.PDB_DIALECT
    elif prof.dialect=='CHARMM':
      prof.processor.dialect=conop.CHARMM_DIALECT
  reader=PDBReader(filename, prof)
  reader.read_seqres=seqres
  try:
    if load_multi:
      ent_list=[]
      while reader.HasNext():
        ent=mol.CreateEntity()
        reader.Import(ent, restrict_chains)
        if prof.processor:
          prof.processor.Process(ent)
        ent_list.append(ent)
      if len(ent_list)==0:
        raise IOError("File '%s' doesn't contain any entities" % filename)
      return ent_list
    else:
      ent=mol.CreateEntity()
      if reader.HasNext():
        reader.Import(ent, restrict_chains)
        if prof.processor:
          prof.processor.Process(ent)
      else:
        raise IOError("File '%s' doesn't contain any entities" % filename)
      if seqres:
        return ent, reader.seqres
      return ent
  except:
    raise

def SavePDB(models, filename, dialect=None,  pqr=False, profile='DEFAULT'):
  """
  Save entity or list of entities to disk. If a list of entities is supplied
  the PDB file will be saved as a multi PDB file. Each of the entities is
  wrapped into a MODEL/ENDMDL pair.

  If the atom number exceeds 99999, '*****' is used.

  :param models: The entity or list of entities (handles or views) to be saved
  :param filename: The filename
  :type  filename: string
  :raises: IOException if the restrictions of the PDB format are not satisfied
           (with the exception of atom numbers, see above):

             * Chain names with more than one character
             * Atom positions with coordinates outside range [-999.99, 9999.99]
             * Residue names longer than three characters
             * Atom names longer than four characters
             * Numeric part of :class:`ost.mol.ResNum` outside range [-999, 9999] 
             * Alternative atom indicators longer than one character
  """
  if not getattr(models, '__len__', None):
    models=[models]
  if isinstance(profile, str):
    profile=profiles[profile].Copy()
  elif isinstance(profile, IOProfile):
    profile.Copy()
  else:
    raise TypeError('profile must be of type string or IOProfile, '+\
                    'instead of %s'%type(profile))
  profile.dialect=_override(profile.dialect, dialect)
  writer=PDBWriter(filename, profile)
  writer.SetIsPQR(pqr)
  if len(models)>1:
    writer.write_multi_model=True
  for model in models:
    writer.Write(model)

try:
  from ost import img
  LoadMap = LoadImage
  SaveMap = SaveImage
except ImportError:
  pass

 ## loads several images and puts them in an ImageList
 # \sa \example fft_li.py "View Fourier Transform Example"
def LoadImageList (files):
  image_list=img.ImageList()
  for file in files:
    image=LoadImage(file)
    image_list.append(image)
  return image_list

LoadMapList=LoadImageList

def LoadCHARMMTraj(crd, dcd_file=None, profile='CHARMM',
                   lazy_load=False, stride=1, 
                   dialect=None, detect_swap=True,swap_bytes=False):
  """
  Load CHARMM trajectory file.
  
  :param crd: EntityHandle or filename of the (PDB) file containing the
      structure. The structure must have the same number of atoms as the 
      trajectory
  :param dcd_file: The filename of the DCD file. If not set, and crd is a 
      string, the filename is set to the <crd>.dcd
  :param layz_load: Whether the trajectory should be loaded on demand. Instead 
      of loading the complete trajectory into memory, the trajectory frames are 
      loaded from disk when requested.
  :param stride: The spacing of the frames to load. When set to 2, for example, 
      every second frame is loaded from the trajectory. By default, every frame 
      is loaded.
  :param dialect: The dialect for the PDB file to use. See :func:`LoadPDB`. If 
      set, overrides the value of the profile
  :param profile: The IO profile to use for loading the PDB file. See 
      :doc:`profile`.
  :param detect_swap: if True (the default), then automatic detection of endianess
      is attempted, otherwise the swap_bytes parameter is used
  :param swap_bytes: is detect_swap is False, this flag determines whether bytes
      are swapped upon loading or not
  """
  if not isinstance(crd, mol.EntityHandle):
    if dcd_file==None:
      dcd_file='%s.dcd' % os.path.splitext(crd)[0]    
    crd=LoadPDB(crd, profile=profile, dialect=dialect)

  else:
    if not dcd_file:
      raise ValueError("No DCD filename given")
  return LoadCHARMMTraj_(crd, dcd_file, stride, lazy_load, detect_swap, swap_bytes)

def LoadMMCIF(filename, fault_tolerant=None, calpha_only=None, profile='DEFAULT', remote=False, seqres=False, info=False):
  """
  Load a mmCIF file and return one or more entities. Several options allow to
  customize the exact behaviour of the mmCIF import. For more information on
  these options, see :doc:`profile`.
  
  Residues are flagged as ligand if they are mentioned in a HET record.

  :param fault_tolerant: Enable/disable fault-tolerant import. If set, overrides
     the value of :attr:`IOProfile.fault_tolerant`.

  :param remote: If set to True, the method tries to load the pdb from the 
     remote pdb repository www.pdb.org. The filename is then interpreted as the 
     pdb id.

  :rtype: :class:`~ost.mol.EntityHandle` (or tuple if *seqres* or *info* are
          True).

  :param seqres: Whether to read SEQRES records. If True, a
                 :class:`~ost.seq.SequenceList` object is returned as the second
                 item. The sequences in the list are named according to the
                 mmCIF chain name.
                 This feature requires a default
                 :class:`compound library <ost.conop.CompoundLib>`
                 to be defined and accessible via
                 :func:`~ost.conop.GetDefaultLib` or an empty list is returned.

  :param info: Whether to return an info container with the other output.
               If True, a :class:`MMCifInfo` object is returned as last item.

  :raises: :exc:`~ost.io.IOException` if the import fails due to an erroneous
           or non-existent file.
  """
  def _override(val1, val2):
    if val2!=None:
      return val2
    else:
      return val1
  if isinstance(profile, str):
    prof = profiles[profile].Copy()
  else:
    prof = profile.Copy()

  prof.calpha_only=_override(prof.calpha_only, calpha_only)
  prof.fault_tolerant=_override(prof.fault_tolerant, fault_tolerant)

  if remote:
    from ost.io.remote import RemoteGet
    tmp_file = RemoteGet(filename, from_repo='cif')
    filename = tmp_file.name
  
  try:
    ent = mol.CreateEntity()
    reader = MMCifReader(filename, ent, prof)
    reader.read_seqres = seqres
    
    # NOTE: to speed up things, we could introduce a restrict_chains parameter
    #       similar to the one in LoadPDB. Here, it would have to be a list/set
    #       of chain-name-strings.

    #if reader.HasNext():
    reader.Parse()
    if prof.processor:
      prof.processor.Process(ent)
      reader.info.ConnectBranchLinks()
    #else:
    #  raise IOError("File doesn't contain any entities")
    if seqres and info:
      return ent, reader.seqres, reader.info
    if seqres:
      return ent, reader.seqres
    if info:
      return ent, reader.info
    return ent
  except:
    raise

# this function uses a dirty trick: should be a member of MMCifInfoBioUnit
# which is totally C++, but we want the method in Python... so we define it
# here (__init__) and add it as a member to the class. With this, the first
# arguement is the usual 'self'.
# documentation for this function was moved to mmcif.rst,
# MMCifInfoBioUnit.PDBize, since this function is not included in SPHINX.
def _PDBize(biounit, asu, seqres=None, min_polymer_size=None,
            transformation=False, peptide_min_size=10, nucleicacid_min_size=10,
            saccharide_min_size=10):
  if min_polymer_size is not None:
    pdbizer = mol.alg.PDBize(min_polymer_size=min_polymer_size)
  else:
    pdbizer = mol.alg.PDBize(peptide_min_size=peptide_min_size,
                             nucleicacid_min_size=nucleicacid_min_size,
                             saccharide_min_size=saccharide_min_size)

  chains = biounit.GetChainList()
  c_intvls = biounit.GetChainIntervalList()
  o_intvls = biounit.GetOperationsIntervalList()
  ss = seqres
  if not ss:
    ss = seq.CreateSequenceList()
  # create list of operations
  # for cartesian products, operations are stored in a list, multiplied with
  # the next list of operations and re-stored... until all lists of operations
  # are multiplied in an all-against-all manner.
  operations = biounit.GetOperations()
  for i in range(0,len(c_intvls)):
    trans_matrices = geom.Mat4List()
    l_operations = operations[o_intvls[i][0]:o_intvls[i][1]]
    if len(l_operations) > 0:
      for op in l_operations[0]:
        rot = geom.Mat4()
        rot.PasteRotation(op.rotation)
        trans = geom.Mat4()
        trans.PasteTranslation(op.translation)
        tr = geom.Mat4()
        tr = trans * rot
        trans_matrices.append(tr)
      for op_n in range(1, len(l_operations)):
        tmp_ops = geom.Mat4List()
        for o in l_operations[op_n]:
          rot = geom.Mat4()
          rot.PasteRotation(o.rotation)
          trans = geom.Mat4()
          trans.PasteTranslation(o.translation)
          tr = geom.Mat4()
          tr = trans * rot
          for t_o in trans_matrices:
            tp = t_o * tr
            tmp_ops.append(tp)
        trans_matrices = tmp_ops
    # select chains into a view as basis for each transformation
    assu = asu.Select('cname='+','.join(mol.QueryQuoteName(name) \
                                        for name in \
                                        chains[c_intvls[i][0]:c_intvls[i][1]]))
    pdbizer.Add(assu, trans_matrices, ss)
  pdb_bu = pdbizer.Finish(transformation)
  if transformation:
    return pdb_bu, pdb_bu.GetTransform().GetMatrix()
  return pdb_bu

MMCifInfoBioUnit.PDBize = _PDBize



import sys
import unittest
from ost import *
from ost import settings
from ost.bindings import kclust

class TestkClustBindings(unittest.TestCase):
  
  def testkClustExec(self):

    seq_list=io.LoadSequenceList('testfiles/multiple.fasta')

    clusters40=kclust.kClust(seq_list,clustering_thresh=40)
    clusters90=kclust.kClust(seq_list,clustering_thresh=90)

    self.assertEqual(len(clusters40),16)
    self.assertEqual(len(clusters90),27)

  def testkClustParseOutput(self):

    representative_ids=['Q5KTS5.1','P46331.2','P66776.1','P42317.2','P50199.1',
                        'Q59787.1','Q9WYG0.1','P50197.1','P50198.1','Q7Z4W1.2',
                        'Q04520.1']

    cluster_sizes=[3,3,4,1,7,3,2,1,1,3,1]

    result=kclust._ParseOutput('testfiles')

    self.assertEqual(len(result),11)

    for c, r_id in zip(result, representative_ids):
      self.assertEqual(c.representative_id, r_id)
    for c, s in zip(result, cluster_sizes):
      self.assertEqual(len(c.sequences),s)

if __name__ == "__main__":
  # test if kClust is available on system, otherwise ignore tests
  try:
    blastpath=settings.Locate(('kClust'))
  except(settings.FileNotFound):
    print("Could not find kClust executable: ignoring unit tests")
    sys.exit(0)
  from ost import testutils
  testutils.RunTests()



pdb_name='sdh.pdb'
if len(sys.argv)>1:
  pdb_name=sys.argv[1]
ent=io.LoadEntity(pdb_name)

vv=ent.Select('peptide=true and aname=CA,N,C,O')
gradient=gfx.Gradient()
gradient.SetColorAt(1.0, gfx.Color(1.0, 0.0, 0.0, 1.0))
gradient.SetColorAt(0.5, gfx.Color(0.0, 1.0, 0.0, 1.0))
gradient.SetColorAt(0.0, gfx.Color(0.0, 0.0, 1.0, 1.0))

go=gfx.Entity(os.path.splitext(os.path.basename(pdb_name))[0], vv)
go.ColorBy('abfac', gradient)
scene.Add(go)
scene.center=go.center